@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_WARY_H
-#define POLYMAKE_INTERNAL_WARY_H
+#pragma once
 
 #include "polymake/internal/iterators.h"
 
@@ -132,7 +131,6 @@ namespace polymake {
    using pm::is_wary;
 }
 
-#endif // POLYMAKE_INTERNAL_WARY_H
 
 // Local Variables:
 // mode:C++
