@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_OPERATIONS_H
-#define POLYMAKE_INTERNAL_OPERATIONS_H
+#pragma once
 
 #include "polymake/internal/operations_basic_defs.h"
 #include "polymake/internal/iterators.h"
@@ -178,13 +178,13 @@ struct mul_from_left : mul_impl<RightRef, LeftRef> {
    template <typename L, typename R>
    decltype(auto) operator() (L&& l, R&& r) const
    {
-      return r * l;
+      return r*l;
    }
 
    template <typename L, typename R>
    void assign(L&& l, R&& r) const
    {
-      l = r * l;
+      l = r*l;
    }
 };
 
@@ -1105,22 +1105,20 @@ void accumulate_in(Iterator&& src, const Operation& op_arg, Object&& x)
 }
 
 template <typename Container, typename Operation>
-typename object_traits<typename Container::value_type>::persistent_type
-accumulate(const Container& c, const Operation& op_arg)
+auto accumulate(const Container& c, const Operation& op_arg)
 {
-   typedef typename object_traits<typename Container::value_type>::persistent_type Object;
-   if (c.empty()) return Object();
-   auto src=entire_range(c);
-   Object x=*src;
+   using elem_t = typename object_traits<typename Container::value_type>::persistent_type;
+   if (c.empty()) return elem_t{};
+   auto src = entire_range(c);
+   elem_t x{*src};
    accumulate_in(++src, op_arg, x);
    return x;
 }
 
 template <typename Container>
-typename object_traits<typename Container::value_type>::persistent_type
-average(const Container& c)
+auto average(const Container& c)
 {
-   return accumulate(c, BuildBinary<operations::add>()) / c.size();
+   return accumulate(c, BuildBinary<operations::add>()) / static_cast<matching_primitive_number_t<Container>>(c.size());
 }
 
 namespace operations {
@@ -1323,7 +1321,7 @@ public:
 template <typename IteratorRef>
 struct index2element {
    typedef IteratorRef argument_type;
-   typedef const int result_type;
+   typedef Int result_type;
    result_type operator() (argument_type it) const { return it.index(); }
 };
 
@@ -1533,7 +1531,6 @@ struct is_default_constructible<pm::operations::composed11<InnerOperation, Outer
 
 #endif // __clang__
 
-#endif // POLYMAKE_INTERNAL_OPERATIONS_H
 
 // Local Variables:
 // mode:C++
