@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_POLYTOPE_BENEATH_BEYOND_H
-#define POLYMAKE_POLYTOPE_BENEATH_BEYOND_H
+#pragma once
 
 #include "polymake/Array.h"
 #include "polymake/Set.h"
@@ -46,7 +45,6 @@ public:
 } } // end namespace polymake
 
 
-#endif // POLYMAKE_POLYTOPE_BENEATH_BEYOND_H
 
 // Local Variables:
 // mode:C++
