@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2022
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_TOPAZ_BISTELLAR_COMPLEX_H
-#define POLYMAKE_TOPAZ_BISTELLAR_COMPLEX_H
+#pragma once
 
 #include "polymake/topaz/complex_tools.h"
 #include "polymake/FacetList.h"
@@ -207,7 +206,6 @@ public:
 
 } }
 
-#endif // POLYMAKE_TOPAZ_BISTELLAR_COMPLEX_H
 
 // Local Variables:
 // mode:C++
