@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -20,23 +21,23 @@
 
 namespace polymake { namespace polytope {
 
-bool isomorphic(perl::Object p1, perl::Object p2)
+bool isomorphic(BigObject p1, BigObject p2)
 {
    const IncidenceMatrix<> M1=p1.give("RAYS_IN_FACETS"), M2=p2.give("RAYS_IN_FACETS");
    return graph::isomorphic(M1, M2);
 }
 
-bool is_self_dual(perl::Object p)
+bool is_self_dual(BigObject p)
 {
    const IncidenceMatrix<> M=p.give("RAYS_IN_FACETS");
    return graph::isomorphic(M, T(M));
 }
 
-std::pair< Array<int>, Array<int> >
-find_facet_vertex_permutations(perl::Object p1, perl::Object p2)
+optional<std::pair<Array<Int>, Array<Int>>>
+find_facet_vertex_permutations(BigObject p1, BigObject p2)
 {
-   const IncidenceMatrix<> M1=p1.give("RAYS_IN_FACETS"), M2=p2.give("RAYS_IN_FACETS");
-   return graph::find_row_col_permutation(M1,M2);
+   const IncidenceMatrix<> M1 = p1.give("RAYS_IN_FACETS"), M2 = p2.give("RAYS_IN_FACETS");
+   return graph::find_row_col_permutation(M1, M2);
 }
 
 UserFunction4perl("# @category Comparing"
@@ -58,11 +59,10 @@ UserFunction4perl("# @category Comparing"
                   "# The facet permutation is the first component, the vertex permutation is the second component of the return value."
                   "# "
                   "# Only the combinatorial isomorphism is considered."
-                  "# If the polytopes are not isomorphic, an exception is thrown."
                   "# @param Cone P1 the first cone/polytope"
                   "# @param Cone P2 the second cone/polytope"
-                  "# @return Pair<Array<Int>, Array<Int>> the facet and the vertex permutations"
-                  "# @example [prefer cdd] To print the vertex permutation that maps the 3-simplex to its mirror image, type this:"
+                  "# @return Pair<Array<Int>, Array<Int>> the facet and the vertex permutations, or undef if polytopes are not isomorphic"
+                  "# @example [prefer cdd] [require bundled:cdd] To print the vertex permutation that maps the 3-simplex to its mirror image, type this:"
                   "# > $p = find_facet_vertex_permutations(simplex(3),scale(simplex(3),-1));"
                   "# > print $p->first;"
                   "# | 1 2 3 0",
