@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-http://www.polymake.org
+/* Copyright (c) 1997-2022
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -19,6 +20,18 @@ GNU General Public License for more details.
 
 namespace polymake { namespace polytope {
 
+namespace {
+
+template <typename TMatrix>
+decltype(auto) first_non_ray(const GenericMatrix<TMatrix>& V)
+{
+   for (auto r = entire(rows(V)); !r.at_end(); ++r) {
+      if (!is_zero(r->front()))
+         return *r;
+   }
+   throw std::runtime_error("all VERTICES | POINTS are rays");
+}
+
 // translate all entries of the matrix that do not have 0 as first coordinate (i.e. non-rays) by t
 template <typename Scalar>
 Matrix<Scalar> translate_non_rays(const Matrix<Scalar>& M, const Vector<Scalar>& t)
@@ -34,8 +47,8 @@ Matrix<Scalar> translate_non_rays(const Matrix<Scalar>& M, const Vector<Scalar>&
    return tM;
 }
 
-template<typename Scalar>
-Vector<Scalar> separate_strong(perl::Object p1, perl::Object p2)
+template <typename Scalar>
+Vector<Scalar> separate_strong(BigObject p1, BigObject p2)
 {
    const Matrix<Scalar>
       V = p1.give("VERTICES | POINTS"),
@@ -43,14 +56,9 @@ Vector<Scalar> separate_strong(perl::Object p1, perl::Object p2)
       L1 = p1.give("LINEALITY_SPACE | INPUT_LINEALITY"),
       L2 = p2.give("LINEALITY_SPACE | INPUT_LINEALITY");
 
-   int i=0; //index of the first non-ray in V
-   for (auto r = entire(rows(V)); !r.at_end(); ++r, ++i)
-      if (!is_zero((*r)[0])) break;
-
-   //origin needs to be on the positive side of the separating plane in order for the LP to work.
-   //translate the whole thing s.t. v_0 is the origin
-   Vector<Scalar> t = V[i];
-   t[0]=0;
+   // origin needs to be on the positive side of the separating plane in order for the LP to work.
+   // translate the whole thing s.t. v_0 is the origin
+   Vector<Scalar> t = zero_value<Scalar>() | first_non_ray(V).slice(range_from(1));
 
    const Matrix<Scalar> ineqs( ( (translate_non_rays(V,t) / -translate_non_rays(W,t)) | //V is on the positive, W on the negative side of the hyperplane
                                  ( -ones_vector<Scalar>(V.rows()+W.rows()))) //vz - eps >= 0 and wz + eps <= 0
@@ -62,26 +70,21 @@ Vector<Scalar> separate_strong(perl::Object p1, perl::Object p2)
    if (L2.rows()) eqs /= L2;
    eqs = eqs | zero_vector<Scalar>();
 
-   const Vector<Scalar> obj = unit_vector<Scalar>(V.cols()+1, V.cols());//maximize eps
+   const auto S = solve_LP(ineqs, eqs, unit_vector<Scalar>(V.cols()+1, V.cols()), true);  // maximize eps
+   if (S.status != LP_status::valid || S.objective_value == 0)
+      throw infeasible();  // the only separating plane is a weak one
 
-   to_interface::solver<Scalar> S;
-   Vector<Scalar> P, sep_hyp;
-
-   Vector<Scalar> sol;
-   // exception is caught in the wrapper
-   std::pair<Scalar,Vector<Scalar>> sol_pair = S.solve_lp(ineqs, eqs, obj, true);
-   if (sol_pair.first == 0) throw infeasible(); //the only separating plane is a weak one
-   sol = sol_pair.second.slice(0, V.cols());
+   Vector<Scalar> sol = S.solution.slice(sequence(0, V.cols()));
 
    // translate sol back to the original polytope position
-   for (int i=1; i<sol.dim(); ++i){
+   for (Int i = 1; i < sol.dim(); ++i) {
       if (!is_zero(sol[i])) {
-         t[i] -= sol[0]/sol[i];
+         t[i] -= sol[0] / sol[i];
          break;
       }
    }
-   t[0]=1;
-   sol[0] = -(sol * t - sol[0]);
+   t[0] = 1;
+   sol[0] = -(sol*t - sol[0]);
    return sol;
 }
 
@@ -89,7 +92,7 @@ Vector<Scalar> separate_strong(perl::Object p1, perl::Object p2)
 // t is an inner point of p1 that is not a vertex of p2
 // called from separate_weak
 template <typename Scalar>
-Vector<Scalar> separate_weak(perl::Object p1, perl::Object p2, Vector<Scalar> t)
+Vector<Scalar> separate_weak(BigObject p1, BigObject p2, Vector<Scalar> t)
 {
    const Matrix<Scalar>
       V = p1.give("VERTICES | POINTS"),
@@ -103,39 +106,33 @@ Vector<Scalar> separate_weak(perl::Object p1, perl::Object p2, Vector<Scalar> t)
 
    // origin needs to be on the positive side of the separating plane and must not be a vertex of p1
    // in order for the LP to work. translate the whole thing s.t. the origin lies in the interior of p1
-   t[0]=0;
-
-   int i=0; //index of the first non-ray in V
-   for (auto r = entire(rows(V)); !r.at_end(); ++r, ++i)
-      if (!is_zero((*r)[0])) break;
-
+   t[0] = zero_value<Scalar>();
 
    // obj must not be a vertex of p1 or p2, so we choose an inner point of the translated polytope.
-   const Vector<Scalar> obj = ones_vector<Scalar>(1) | ((V[i] - t)/2).slice(1);
+   const Vector<Scalar> obj = one_value<Scalar>() | ((first_non_ray(V) - t)/2).slice(range_from(1));
 
    const Matrix<Scalar> ineqs( ( translate_non_rays(V,t) / -translate_non_rays(W,t)) //V is on the positive, W on the negative side of the hyperplane
-                               / (2*ones_vector<Scalar>(1) | obj.slice(1))); //obj*z >= -1 to ensure boundedness
+                               / (2 | obj.slice(range_from(1)))); //obj*z >= -1 to ensure boundedness
 
    Matrix<Scalar> eqs(0, V.cols());
    if (L1.rows()) eqs /= L1;
    if (L2.rows()) eqs /= L2;
 
-   to_interface::solver<Scalar> S;
-   Vector<Scalar> P, sep_hyp;
+   const auto S = solve_LP(ineqs, eqs, obj, false);  // minimize obj*z
+   if (S.status != LP_status::valid)
+      throw infeasible();
 
-   Vector<Scalar> sol;
-   // exception is caught in separate_weak
-   sol = S.solve_lp(ineqs, eqs, obj, false).second; //minimize obj*z
+   Vector<Scalar> sol = S.solution;
 
    // translate sol back to the original polytope position
-   for (int i=1;i<sol.dim(); ++i){
+   for (Int i = 1; i < sol.dim(); ++i) {
       if (!is_zero(sol[i])) {
-         t[i] -= sol[0]/sol[i];
+         t[i] -= sol[0] / sol[i];
          break;
       }
    }
-   t[0]=1;
-   sol[0] = -(sol * t - sol[0]);
+   t[0] = one_value<Scalar>();
+   sol[0] = -(sol*t - sol[0]);
    return sol;
 }
 
@@ -147,7 +144,7 @@ Vector<Scalar> separate_weak(perl::Object p1, perl::Object p2, Vector<Scalar> t)
 // of the polytope that is supposed to lie on the positive side of the plane.
 // in case both polytopes are low dimensional, we have to take extra care (see below)
 template <typename Scalar>
-Vector<Scalar> separate_weak(perl::Object p1, perl::Object p2)
+Vector<Scalar> separate_weak(BigObject p1, BigObject p2)
 {
    const Matrix<Scalar>
       V = p1.give("VERTICES | POINTS"),
@@ -171,59 +168,58 @@ Vector<Scalar> separate_weak(perl::Object p1, perl::Object p2)
 
    // both are lowdim
 
-   if (V.rows()!=1) {  // p1 is not a point
+   if (V.rows() != 1) {  // p1 is not a point
       try {
          // we need to find an inner point of p1 that is not a vertex of p2.
          // the loop terminates after a maximum of 2 restarts as there is a maximum of
          // 2 vertices of p2 on the line from inn to V[i]
 
-         int i=0;  // index of the first non-ray in V
-         for (auto r = entire(rows(V)); !r.at_end(); ++r, ++i)
-            if (!is_zero((*r)[0])) break;
+         const auto non_ray = first_non_ray(V);
 
          Vector<Scalar> inn = p1.give("REL_INT_POINT");
          for (auto r = entire(rows(W)); !r.at_end(); ) {
             if (*r == inn) {
-               inn = ones_vector<Scalar>(1) | (inn + (V[i]-inn)/2).slice(1);
+               inn = one_value<Scalar>() | (inn+(non_ray - inn)/2).slice(range_from(1));
                r = entire(rows(W));
             }
-            else ++r;
+            else
+               ++r;
          }
 
-         return separate_weak<Scalar>(p1,p2,inn);
-      } catch(infeasible) { }
+         return separate_weak<Scalar>(p1, p2, inn);
+      } catch(const infeasible&) { }
    }
 
-   if (W.rows()!=1) {  // p2 is not a point
+   if (W.rows() != 1) {  // p2 is not a point
       try {
-         int i=0;  // index of the first non-ray in V
-         for (auto r = entire(rows(W)); !r.at_end(); ++r, ++i)
-            if (!is_zero((*r)[0])) break;
+         const auto non_ray = first_non_ray(W);
 
          // maybe the inner point lies on the separating plane!
          // try the same for p1 and p2 swapped:
          Vector<Scalar> inn = p2.give("REL_INT_POINT");
          for (auto r = entire(rows(V)); !r.at_end(); ) {
             if (*r == inn) {
-               inn = ones_vector<Scalar>(1) | (inn + (W[i]-inn)/2).slice(1);
+               inn = one_value<Scalar>() | (inn+(non_ray - inn)/2).slice(range_from(1));
                r = entire(rows(V));
             }
             else ++r;
          }
-         return -separate_weak<Scalar>(p2,p1,inn);
-      } catch (infeasible) { }
+         return -separate_weak<Scalar>(p2, p1, inn);
+      } catch (const infeasible&) { }
    }
 
    // there are only two options left:
    // either both inner points lie on the separating plane, so p1 and p2 lie in a common hyperplane,
    // or they are inseparable.
-   if ((W*T(ker1)).non_zero()) throw infeasible(); //no common plane
+   if ((W*T(ker1)).non_zero()) throw infeasible(); // no common plane
    return ker1[0];
 }
 
+}
+
 // separate two polytopes
 template< typename Scalar>
-Vector<Scalar> separating_hyperplane(perl::Object p1, perl::Object p2, perl::OptionSet options)
+Vector<Scalar> separating_hyperplane(BigObject p1, BigObject p2, OptionSet options)
 {
    bool strong = options["strong"];
    Vector<Scalar> sol;
@@ -233,60 +229,20 @@ Vector<Scalar> separating_hyperplane(perl::Object p1, perl::Object p2, perl::Opt
       else
          sol = separate_weak<Scalar>(p1,p2);
    }
-   catch (infeasible) {
+   catch (const infeasible&) {
       throw std::runtime_error("separating_hyperplane: the given polytopes cannot be separated");
    }
    return sol;
 }
 
-template <typename Scalar>
-bool cone_contains_point(perl::Object p, const Vector<Scalar> & q, perl::OptionSet options)
-{
-   // the LP constructed here is supposed to find a conical/convex combination of the rays/vertices
-   // of cone/polytope p that equals q. if the in_interior flag is set, all coefficients are
-   // required to be strictly positive.
-   Matrix<Scalar> P = p.give("RAYS | INPUT_RAYS");
-
-   Matrix<Scalar> eqs = -q | T(P) ; //sum z_i p_i = q
-   //(if p_i are given in hom.coords, also sum z_i = 1 for all i whose p_i are not rays)
-
-   //lookup prevents computation of this property in case it is not given.
-   Matrix<Scalar> E = p.lookup("LINEALITY_SPACE | INPUT_LINEALITY");
-   if (E.rows()) {
-      // for polytopes, replace first col of E with 0, so we have sum z_i = 1 only for points
-      if (p.isa("Polytope")) E = zero_vector<Scalar>() | E.minor(All,range(1,E.cols()-1));
-
-      eqs = eqs | T(E) | T(-E);
-   }
-   eqs = eqs | zero_vector<Scalar>();
-
-   int n = P.rows();
-   Matrix<Scalar> ineqs = zero_vector<Scalar>(n) | unit_matrix<Scalar>(n) | zero_matrix<Scalar>(n, 2*E.rows()) | -ones_vector<Scalar>(n); //z_i >= eps for rays
-
-   int c = n + 2*E.rows() + 2;
-   ineqs = ineqs / unit_vector<Scalar>(c,c-1) //eps >= 0
-                 / (unit_vector<Scalar>(c,0) - unit_vector<Scalar>(c,c-1)); // eps <= 1
-
-   Vector<Scalar> obj = unit_vector<Scalar>(c,c-1); //maximize eps
-   to_interface::solver<Scalar> S;
-   try {
-      std::pair<Scalar,Vector<Scalar>> sol = S.solve_lp(ineqs, eqs, obj, true);
-      bool in = options["in_interior"];
-      if (sol.first == 0 && in) return false;  // the only separating plane is a weak one
-   }
-   catch (infeasible) {
-      return false;
-   }
-   return true;
-}
 
 // check for separability of a point and a set of points
 template <typename Scalar>
-bool separable(perl::Object p, const Vector<Scalar>& q, perl::OptionSet options)
+bool separable(BigObject p, const Vector<Scalar>& q, OptionSet options)
 {
    bool strong = options["strong"];
 
-   return !cone_contains_point<Scalar>(p, q, perl::OptionSet("in_interior", !strong));
+   return !cone_contains_point<Scalar>(p, q, OptionSet("in_interior", !strong));
 }
 
 FunctionTemplate4perl("cone_contains_point<Scalar> [ is_ordered_field_with_unlimited_precision(type_upgrade<Scalar, Rational>) ](Cone<Scalar>, Vector<Scalar>, {in_interior=>0})");
@@ -304,7 +260,7 @@ UserFunctionTemplate4perl("# @category Geometry"
                           "# @example"
                           "# > $q = cube(2)->VERTICES->row(0);"
                           "# > print separable(cube(2), $q, strong=>0);"
-                          "# | 1",
+                          "# | true",
                           "separable<Scalar>(Cone<type_upgrade<Scalar>>, Vector<type_upgrade<Scalar>>, {strong=>1})");
 
 UserFunctionTemplate4perl("# @category Optimization"
@@ -332,7 +288,7 @@ UserFunctionTemplate4perl("# @category Optimization"
                           "# @param Polytope p1 the first polytope, will be on the positive side of the separating hyperplane"
                           "# @param Polytope p2 the second polytope"
                           "# @option Bool strong If this is set to true, the resulting hyperplane will be strongly separating,"
-                          "#  i.e. it wont touch either of the polytopes. If such a plane does not exist, an exception"
+                          "#  i.e. it won't touch either of the polytopes. If such a plane does not exist, an exception"
                           "#  will be thrown. default: true"
                           "# @return Vector a hyperplane separating //p1// from //p2//\n",
                           "separating_hyperplane<Scalar>(Polytope<type_upgrade<Scalar>>, Polytope<type_upgrade<Scalar>>, {strong=>1})");
