@@ -17,6 +17,12 @@
 	---
 	Copyright (C) 2011 - 2015, Simon Hampe <simon.hampe@googlemail.com>
 
+	---
+	Copyright (c) 2016-2022
+	Ewgenij Gawrilow, Michael Joswig, and the polymake team
+	Technische Universität Berlin, Germany
+	https://polymake.org
+
 	Implements the addition of two morphisms.
 	*/
 
@@ -39,21 +45,20 @@ namespace polymake { namespace tropical {
  * @return Morphism The "refined" morphism
  */
 template <typename Addition>
-perl::Object refine_domain(perl::Object f, perl::Object cycle, bool need_to_refine)
+BigObject refine_domain(BigObject f, BigObject cycle, bool need_to_refine)
 {
-  perl::Object oldDomain = f.give("DOMAIN");
+  BigObject oldDomain = f.give("DOMAIN");
   bool has_matrix = f.exists("MATRIX");
   RefinementResult r = refinement(cycle, oldDomain,  false, !has_matrix,false,need_to_refine,false);
 
-  perl::Object nDomain = r.complex;
+  BigObject nDomain = r.complex;
   if (has_matrix) {
     Matrix<Rational> matrix = f.give("MATRIX");
     Vector<Rational> translate = f.give("TRANSLATE");
-    perl::Object result("Morphism", mlist<Addition>());
-    result.take("DOMAIN") << nDomain;
-    result.take("MATRIX") << matrix;
-    result.take("TRANSLATE") << translate;
-    return result;
+    return BigObject("Morphism", mlist<Addition>(),
+                     "DOMAIN", nDomain,
+                     "MATRIX", matrix,
+                     "TRANSLATE", translate);
   }
 
   Matrix<Rational> rayRep = r.rayRepFromY;
@@ -62,7 +67,7 @@ perl::Object refine_domain(perl::Object f, perl::Object cycle, bool need_to_refi
   Matrix<Rational> vertex_values = f.give("VERTEX_VALUES");
   Matrix<Rational> lineality_values = f.give("LINEALITY_VALUES");
   Matrix<Rational> total_values = T( vertex_values / lineality_values);
-  int target_dim = std::max(vertex_values.cols(), lineality_values.cols());
+  Int target_dim = std::max(vertex_values.cols(), lineality_values.cols());
 
   Matrix<Rational> ndom_vertices = nDomain.give("SEPARATED_VERTICES");
   Matrix<Rational> ndom_lineality = nDomain.give("LINEALITY_SPACE");
@@ -70,29 +75,28 @@ perl::Object refine_domain(perl::Object f, perl::Object cycle, bool need_to_refi
   Matrix<Rational> rValues(0,target_dim);
   Matrix<Rational> lValues(0,target_dim);
 
-  for (int v = 0; v < ndom_vertices.rows(); ++v) {
+  for (Int v = 0; v < ndom_vertices.rows(); ++v) {
     rValues /= total_values * rayRep.row(v);
   }
 
-  for (int l = 0; l < ndom_lineality.rows(); ++l) {
+  for (Int l = 0; l < ndom_lineality.rows(); ++l) {
     lValues /= total_values * linRep.row(l);
   }
 
-  perl::Object result("Morphism", mlist<Addition>());
-  result.take("DOMAIN") << nDomain;
-  result.take("VERTEX_VALUES") << rValues;
-  result.take("LINEALITY_VALUES") << lValues;
-  return result;
+  return BigObject("Morphism", mlist<Addition>(),
+                   "DOMAIN", nDomain,
+                   "VERTEX_VALUES", rValues,
+                   "LINEALITY_VALUES", lValues);
 }
 
 /**
    @brief Computes the sum of two morphisms (which should be defined on the same support)
-   @param perl::Object f A Morphism object
-   @param perl::Object g A Morphism object, whose DOMAIN has the same support as f's DOMAIN and whose image has the same ambient dimension as f's image
-   @return perl::Object A Morphism object representing f+g
+   @param BigObject f A Morphism object
+   @param BigObject g A Morphism object, whose DOMAIN has the same support as f's DOMAIN and whose image has the same ambient dimension as f's image
+   @return BigObject A Morphism object representing f+g
 */
 template <typename Addition>
-perl::Object add_morphisms(perl::Object f, perl::Object g)
+BigObject add_morphisms(BigObject f, BigObject g)
 {
   // First we treat the special case where both are global
   bool f_global = f.exists("MATRIX");
@@ -110,37 +114,34 @@ perl::Object add_morphisms(perl::Object f, perl::Object g)
   }
 
   // First we homogenize where necessary
-  perl::Object fDomain = f.give("DOMAIN");
-  perl::Object gDomain = g.give("DOMAIN");
+  BigObject fDomain = f.give("DOMAIN");
+  BigObject gDomain = g.give("DOMAIN");
 
   // Then compute the common refinement of the domains
   RefinementResult r = refinement(fDomain,gDomain,false,false,false,true,false);
-  perl::Object nDomain = r.complex;
+  BigObject nDomain = r.complex;
 	
   // If the map is given by a matrix, we're done
   if (f_global && g_global) {
-    perl::Object result("Morphism", mlist<Addition>());
-    result.take("MATRIX") << sum_matrix;
-    result.take("TRANSLATE") << sum_translate;
-    result.take("DOMAIN") << nDomain;
-    return result;
+    return BigObject("Morphism", mlist<Addition>(),
+                     "MATRIX", sum_matrix,
+                     "TRANSLATE", sum_translate,
+                     "DOMAIN", nDomain);
   }
 
   // Otherwise, compute values on the new domain
-  perl::Object f_refined = refine_domain<Addition>(f, nDomain, false);
-  perl::Object g_refined = refine_domain<Addition>(g, nDomain, false);
+  BigObject f_refined = refine_domain<Addition>(f, nDomain, false);
+  BigObject g_refined = refine_domain<Addition>(g, nDomain, false);
 
   Matrix<Rational> fref_vert = f_refined.give("VERTEX_VALUES");
   Matrix<Rational> gref_vert = g_refined.give("VERTEX_VALUES");
   Matrix<Rational> fref_lin  = f_refined.give("LINEALITY_VALUES");
   Matrix<Rational> gref_lin  = g_refined.give("LINEALITY_VALUES");
 
-  perl::Object result("Morphism", mlist<Addition>());
-  result.take("DOMAIN") << nDomain;
-  result.take("VERTEX_VALUES") << fref_vert + gref_vert;
-  result.take("LINEALITY_VALUES") << fref_lin + gref_lin;
-
-  return result;
+  return BigObject("Morphism", mlist<Addition>(),
+                   "DOMAIN", nDomain,
+                   "VERTEX_VALUES", fref_vert + gref_vert,
+                   "LINEALITY_VALUES", fref_lin + gref_lin);
 }
 
 UserFunctionTemplate4perl("# @category Morphisms"
